CREATE TABLE Geofence (
	id INTEGER PRIMARY KEY AUTOINCREMENT,
	specId INTEGER UNIQUE NOT NULL,
	borderEventId INTEGER,
	specType INTEGER NOT NULL,
	currentState INTEGER NOT NULL,
	onBorderLocates INTEGER NOT NULL,
	action CHAR(1024),
	componentName CHAR(1024) NOT NULL);


CREATE TABLE CircularSpec (
	id INTEGER PRIMARY KEY AUTOINCREMENT,
	latitude FLOAT NOT NULL,
	longitude FLOAT NOT NULL,
	radius FLOAT NOT NULL,
	crossingTypes INTEGER NOT NULL
);


CREATE TABLE BorderEvent (
	id INTEGER PRIMARY KEY AUTOINCREMENT,
	firetime INTEGER NOT NULL,
	latitude FLOAT NOT NULL,
	longitude FLOAT NOT NULL,
	accuracy FLOAT NOT NULL,
	crossingType INTEGER NOT NULL,
	isInitialState INTEGER NOT NULL
);

CREATE TRIGGER DeleteGeofence AFTER DELETE ON Geofence
   BEGIN
     DELETE FROM CircularSpec WHERE CircularSpec.id = old.specId;
     DELETE FROM BorderEvent WHERE BorderEvent.id = old.borderEventId;
   END;

CREATE TRIGGER UpdateGeofenceBorderEvent AFTER UPDATE OF borderEventId ON Geofence 
     WHEN NOT old.borderEventId = new.borderEventId 
          OR new.borderEventId IS NULL
   BEGIN
     DELETE FROM BorderEvent WHERE BorderEvent.id = old.borderEventId;
   END;

CREATE TABLE SpecType (
	id INTEGER PRIMARY KEY,
	name CHAR(128)
);

INSERT INTO SpecType VALUES (1, 'CircularSpec');

CREATE TABLE StateType (
	val INTEGER PRIMARY KEY,
	name CHAR(128)
);

INSERT INTO StateType VALUES (1, 'ENTER');
INSERT INTO StateType VALUES (2, 'EXIT');

CREATE VIEW CircularGeofence AS SELECT 
     Geofence.id AS id,
     Geofence.currentState AS currentState,
     Geofence.onBorderLocates AS onBorderLocates,
     CircularSpec.latitude AS specLatitude,
     CircularSpec.longitude AS specLongitude,
     CircularSpec.radius AS specRadius,
     CircularSpec.crossingTypes AS crossingTypes,
     Geofence.action AS action,
     Geofence.componentName AS componentName,
     BorderEvent.firetime AS eventTime,
     BorderEvent.latitude AS eventLatitude,
     BorderEvent.longitude AS eventLongitude,
     BorderEvent.accuracy AS eventAccuracy,
     BorderEvent.crossingType AS eventCrossingType,
     BorderEvent.isInitialState AS eventIsInitialState 
   FROM Geofence JOIN CircularSpec ON Geofence.specId = CircularSpec.id
                 LEFT JOIN BorderEvent on Geofence.borderEventId = BorderEvent.id 
   WHERE Geofence.specType = 1;

